;; events.scm: DOM events exports and implementation for SDOM
;; Copyright (C) 2004 Julian Graham

;; SDOM is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2 of the License, or
;; (at your option) any later version.
;;
;; SDOM is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with SDOM; if not, write to the Free Software
;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(define-module (sxml sdom-events)
  #:use-module (sxml sdom)
  #:export     (sdom:events-enabled
		sdom:dispatch-event))

(define sdom:events-enabled #t)

(define sdom:event-structure
  `(sdom:event 
    (@ (! type ())
       (! target ())
       (! current-target ())
       (! event-phase ())
       (! bubbles ())
       (! cancelable ())
       (! time-stamp ())
       (! namespace-uri ()))
    (* (sdom:event-load ,#f ,#f (,sdom:node-type-document
				       ,sdom:node-type-element))
	     (sdom:event-unload ,#f, #f (,sdom:node-type-document
					 ,sdom:node-type-element))
	     (sdom:event-abort ,#t ,#f (,sdom:node-type-element))
	     (sdom:event-error ,#t ,#f (,sdom:node-type-element))
	     (sdom:event-select ,#t ,#f (,sdom:node-type-element))
	     (sdom:event-change ,#t ,#f (,sdom:node-type-element))
	     (sdom:event-submit ,#t ,#t (,sdom:node-type-element))
	     (sdom:event-reset ,#t ,#t (,sdom:node-type-element))
	     (sdom:event-resize ,#t ,#f (,sdom:node-type-document
					 ,sdom:node-type-element))
	     (sdom:event-scroll ,#t ,#f (,sdom:node-type-document
					 ,sdom:node-type-element)))
   (sdom:custom-event)
    (sdom:ui-event 
     (@ (! view ())
	(! detail ()))
     (* (sdom:event-dom-activate ,#t ,#t (,sdom:node-type-element))
	      (sdom:event-dom-focus-in ,#t ,#f (,sdom:node-type-element))
	      (sdom:event-dom-focus-out ,#t, #f (,sdom:node-type-element)))
     (sdom:text-event 
      (@ (! data ()))
      (* (sdom:event-text-input ,#t ,#t (,sdom:node-type-element))))
     (sdom:mouse-event 
      (@ (! screen-x ())
	 (! screen-y ())
	 (! client-x ())
	 (! client-y ())
	 (! ctrl-key ())
	 (! shift-key ())
	 (! alt-key ())
	 (! meta-key ())
	 (! button ())
	 (! related-target ()))
      (* (sdom:event-click ,#t ,#t (,sdom:node-type-element))
	       (sdom:event-mousedown ,#t ,#t (,sdom:node-type-element))
	       (sdom:event-mouseup ,#t ,#t (,sdom:node-type-element))
	       (sdom:event-mouseover ,#t ,#t (,sdom:node-type-element))
	       (sdom:event-mousemove ,#t ,#t (,sdom:node-type-element))
	       (sdom:event-mouseout ,#t ,#t (,sdom:node-type-element)))
     (sdom:keyboard-event 
      (@ (! key-identifier ())
	 (! key-location ())
	 (! ctrl-key ())
	 (! shift-key ())
	 (! alt-key ())
	 (! meta-key ()))
      (* (sdom:event-keydown ,#t ,#t (,sdom:node-type-element))
	       (sdom:event-keyup ,#t ,#t (,sdom:node-type-element))))
     (sdom:mutation-event 
      (@ (! related-node ())
	 (! prev-value ())
	 (! new-value ())
	 (! attr-name ())
	 (! attr-change ()))
      (* (sdom:event-dom-subtree-modified 
		,#t 
		,#f
		(,sdom:node-type-document
		 ,sdom:node-type-document-fragment
		 ,sdom:node-type-element
		 ,sdom:node-type-attr))
	 (sdom:event-dom-node-inserted 
	  ,#t 
	  ,#f
	  (,sdom:node-type-element
	   ,sdom:node-type-attr
	   ,sdom:node-type-text
	   ,sdom:node-type-comment
	   ,sdom:node-type-cdata-section
	   ,sdom:node-type-document-type
	   ,sdom:node-type-entity-reference
	   ,sdom:node-type-processing-instruction))
	 (sdom:event-dom-node-removed ,#t ,#f,
				      (,sdom:node-type-element
				       ,sdom:node-type-attr
				       ,sdom:node-type-text
				       ,sdom:node-type-comment
				       ,sdom:node-type-cdata-section
				       ,sdom:node-type-document-type
				       ,sdom:node-type-entity-reference
				       ,sdom:node-type-processing-instruction))
	 (sdom:event-dom-node-removed-from-document 
	  ,#f 
	  ,#f
	  (,sdom:node-type-element
	   ,sdom:node-type-attr
	   ,sdom:node-type-text
	   ,sdom:node-type-comment
	   ,sdom:node-type-cdata-section
	   ,sdom:node-type-document-type
	   ,sdom:node-type-entity-reference
	   ,sdom:node-type-processing-instruction))
	 (sdom:event-dom-node-inserted-into-document 
	  ,#f 
	  ,#f
	  (,sdom:node-type-element
	   ,sdom:node-type-attr
	   ,sdom:node-type-text
	   ,sdom:node-type-comment
	   ,sdom:node-type-cdata-section
	   ,sdom:node-type-document-type
	   ,sdom:node-type-entity-reference
	   ,sdom:node-type-processing-instruction))
	 (sdom:event-dom-attr-modified ,#t ,#f (,sdom:node-type-element))
	 (sdom:event-dom-character-data-modified ,#t ,#f
						 (,sdom:node-type-document
						  ,sdom:node-type-element)))
     (sdom:mutation-name-event 
      (@ (! prev-namespace-uri ())
	  (! prev-node-name ()))
      (* (sdom:event-dom-element-name-changed ,#t ,#f)
		(sdom:event-dom-attribute-name-changed ,#t ,#f))))))))

(define sdom:dispatch-event
  (lambda (target event . data)
    (let ((event-queue-annotation (cadr (cadr (cadr sdom-tree)))))   
      (if (equal? (length (cadr event-queue-annotation)) 0)
	  (set-cdr! event-queue-annotation (cons (cons event-data '()) '()))
	  (let ((event-queue (cadr event-queue-annotation)))
	    (set-cdr! (last-pair event-queue) (cons event-data '())))))))


